@@ -6,20 +6,14 @@ var ufuzz = require("./ufuzz");
 var UglifyJS = require("..");
 
 function try_beautify(code) {
-    var beautified;
-    try {
-        beautified = UglifyJS.minify(code, {
-            fromString: true,
-            compress: false,
-            mangle: false,
-            output: {
-                beautify: true,
-                bracketize: true
-            }
-        });
-    } catch (ex) {
-        beautified = { error: ex };
-    }
+    var beautified = UglifyJS.minify(code, {
+        compress: false,
+        mangle: false,
+        output: {
+            beautify: true,
+            bracketize: true
+        }
+    });
     if (beautified.error) {
         console.log("// !!! beautify failed !!!");
         console.log(beautified.error.stack);
@@ -31,17 +25,10 @@ function try_beautify(code) {
 }
 
 function test(original, estree, description) {
-    var transformed;
-    try {
-        transformed = UglifyJS.minify(estree, {
-            fromString: true,
-            compress: false,
-            mangle: false,
-            spidermonkey: true
-        });
-    } catch (ex) {
-        transformed = { error: ex };
-    }
+    var transformed = UglifyJS.minify(UglifyJS.AST_Node.from_mozilla_ast(estree), {
+        compress: false,
+        mangle: false
+    });
     if (transformed.error || original !== transformed.code) {
         console.log("//=============================================================");
         console.log("// !!!!!! Failed... round", round);
@@ -65,14 +52,13 @@ var num_iterations = ufuzz.num_iterations;
 for (var round = 1; round <= num_iterations; round++) {
     process.stdout.write(round + " of " + num_iterations + "\r");
     var code = ufuzz.createTopLevelCode();
-    var uglified = {
-        ast: UglifyJS.parse(code),
-        code: UglifyJS.minify(code, {
-            fromString: true,
-            compress: false,
-            mangle: false
-        }).code
-    };
+    var uglified = UglifyJS.minify(code, {
+        compress: false,
+        mangle: false,
+        output: {
+            ast: true
+        }
+    });
     test(uglified.code, uglified.ast.to_mozilla_ast(), "AST_Node.to_mozilla_ast()");
     try {
         test(uglified.code, acorn.parse(code), "acorn.parse()");
